----------------------------------------------------------------------------------
-- Company: Digilent Ro
-- Engineer: Cristian Fatu
-- 
-- Create Date:    2014 March 14  
-- Design Name: 
-- Module Name:    Top - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created, using old Top.vhd file created by Elod Gyorgy
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.math_real.all;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity Top is
	Port (
		SYSCLK_IP : in  STD_LOGIC;
		SYSCLK_IN : in  STD_LOGIC;	

		LCD_R_O : out  STD_LOGIC_VECTOR (7 downto 0);
		LCD_G_O : out  STD_LOGIC_VECTOR (7 downto 0);
		LCD_B_O : out  STD_LOGIC_VECTOR (7 downto 0);
		
		
		LCD_CLK_O : out  STD_LOGIC;
		LCD_DE_O : out  STD_LOGIC;
		LCD_HSYNC_O : out STD_LOGIC;
		LCD_VSYNC_O : out STD_LOGIC;
		LCD_BKLT_O : out  STD_LOGIC;
		LCD_DISP_O : out  STD_LOGIC
			  
		);

end Top;

architecture Behavioral of Top is
	
	constant H_MAX : NATURAL := 889;
	constant V_MAX : NATURAL := 513;
	

	signal SysConCLK : std_logic; --buffered input clock that drives this unit
	signal clk100, clk50, clk20 : std_logic;
	
	
	COMPONENT dcm_hmi
	PORT
	 (-- Clock in ports
	  CLK_IN1           : in     std_logic;
	  -- Clock out ports
	  clk100          : out    std_logic;
	  clk20          : out    std_logic;
	  clk50          : out    std_logic;
	  -- Status and control signals
	  RESET             : in     std_logic;
	  LOCKED            : out    std_logic
	 );
	END COMPONENT;
		
	COMPONENT VideoTimingContentCtl
	PORT(
		PCLK_I : in  STD_LOGIC; 
		VDE_O : out STD_LOGIC; --data enable for pixel bus
		HS_O : out STD_LOGIC;
		VS_O : out STD_LOGIC;


		R_O : out  STD_LOGIC_VECTOR (7 downto 0);
		G_O : out  STD_LOGIC_VECTOR (7 downto 0);
		B_O : out  STD_LOGIC_VECTOR (7 downto 0)
		);
	
	END COMPONENT;
	
	COMPONENT TFTSigCtl
	PORT(
		CLK_I : in STD_LOGIC;		
		CLK_O : out  STD_LOGIC;
		DISP_O : out  STD_LOGIC;
		BKLT_O : out  STD_LOGIC --PWM backlight control
		);
	
	END COMPONENT;

	
begin

	IBUFGDS_inst : IBUFGDS
   generic map (
      IBUF_LOW_PWR => TRUE, -- Low power (TRUE) vs. performance (FALSE) setting for refernced I/O standards
      IOSTANDARD => "DEFAULT")
   port map (
      O => SysConCLK,  -- Clock buffer output
      I => SYSCLK_IP,  -- Diff_p clock buffer input
      IB => SYSCLK_IN -- Diff_n clock buffer input
   );
	
	DCM_inst : dcm_hmi
  port map
   (-- Clock in ports
    CLK_IN1 => SysConCLK,
    -- Clock out ports
    CLK100 => clk100,
    CLK20 => clk20,
    CLK50 => clk50,
    -- Status and control signals
    RESET  => '0',
    LOCKED => open);
	

	Inst_VideoTimingContentCtl: VideoTimingContentCtl PORT MAP(
		PCLK_I => clk50,
		VDE_O => LCD_DE_O,
		HS_O => LCD_HSYNC_O,
		VS_O => LCD_VSYNC_O ,

		R_O => LCD_R_O,
		G_O => LCD_G_O,
		B_O => LCD_B_O
	);
	
	Inst_TFTSigCtl: TFTSigCtl PORT MAP(
		CLK_I=> clk50,
		CLK_O => LCD_CLK_O,
		DISP_O => LCD_DISP_O,
		BKLT_O => LCD_BKLT_O
	);
	

	
end Behavioral;

