----------------------------------------------------------------------------------
-- Company: Digilent Ro
-- Engineer: Cristian Fatu
-- 
-- Create Date:    2014 March 14 
-- Design Name: 
-- Module Name:    VideoTimingContentCtl - Behavioral 
-- Project Name:
-- Target Devices: 
-- Tool versions: 
-- Description: VideoTimingContentCtl generates the proper synchronization signals and 
-- it also generates video content.
--
-- Dependencies: digilent.Video
--
-- Revision: 
-- Revision 0.01 - File Created, using old VideoTimingCtl.vhd file created by Elod Gyorgy
-- Revision 0.02 - File updated to be used with Vivado
-- Additional Comments: 
-- 
----------------------------------------------------------------------------------
library IEEE;
Library UNISIM;
use UNISIM.vcomponents.all;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;


entity VideoTimingContentCtl is
	Port (
			  PCLK_I : in  STD_LOGIC; 
			  VDE_O : out STD_LOGIC; 
			  HS_O : out STD_LOGIC;
			  VS_O : out STD_LOGIC;
			  
				R_O : out  STD_LOGIC_VECTOR (7 downto 0);
				G_O : out  STD_LOGIC_VECTOR (7 downto 0);
				B_O : out  STD_LOGIC_VECTOR (7 downto 0)	
			  );
			  
end VideoTimingContentCtl;

architecture Behavioral of VideoTimingContentCtl is

	--horizontal constants
		constant H_800_480p_S : NATURAL := 1;		--sync
		constant H_800_480p_FP : NATURAL := 1; 	--front porch
		constant H_800_480p_AV : NATURAL := 800; 	--active video
		--constant H_800_480p_BP : NATURAL := 88;	--back porch
        constant H_800_480p_BP : NATURAL := 80;	--back porch
		constant H_MAX : NATURAL := 889;
	--vertical constants
		constant V_800_480p_S : NATURAL := 3;		--sync
		constant V_800_480p_FP : NATURAL := 1; 	--front porch
		constant V_800_480p_AV : NATURAL := 480; 	--active video
		constant V_800_480p_BP : NATURAL := 32;	--back porch
		constant V_MAX : NATURAL := 513;
		
		constant H_800_480p_AV_FP : NATURAL := H_800_480p_AV + H_800_480p_FP;
		constant H_800_480p_AV_FP_S : NATURAL := H_800_480p_AV + H_800_480p_FP + H_800_480p_S;
		constant H_800_480p_AV_FP_S_BP : NATURAL := H_800_480p_AV + H_800_480p_FP + H_800_480p_S + H_800_480p_BP;

		constant V_800_480p_AV_FP : NATURAL := V_800_480p_AV + V_800_480p_FP;
		constant V_800_480p_AV_FP_S : NATURAL := V_800_480p_AV + V_800_480p_FP + V_800_480p_S;
		constant V_800_480p_AV_FP_S_BP : NATURAL := V_800_480p_AV + V_800_480p_FP + V_800_480p_S + V_800_480p_BP;
		
		constant H_800_480p_POL : BOOLEAN := false; -- negative polarity
		constant V_800_480p_POL : BOOLEAN := false; -- negative polarity


    	signal HCnt : NATURAL range 0 to H_MAX - 1:= 0; --horizontal counter
	   signal VCnt : NATURAL range 0 to V_MAX - 1:= 0; --vertical counter
	signal red_reg : std_logic_vector(7 downto 0) := "00000000";
	signal green_reg : std_logic_vector(7 downto 0) := "00000000";
	signal blue_reg : std_logic_vector(7 downto 0) := "00000000";

	signal hs, vs: std_logic; -- horizontal/vertical sync
	signal vde : std_logic; -- data enable

begin

		
process (PCLK_I)
begin
	if Rising_Edge(PCLK_I) then

			--pixel/line counters and video data enable
			if (HCnt = H_800_480p_AV_FP_S_BP - 1) then
				HCnt <= 0;
				if (VCnt = V_800_480p_AV_FP_S_BP - 1) then
					VCnt <= 0;
				else
					VCnt <= VCnt + 1;
				end if;
			else
				HCnt <= HCnt + 1;
			end if;
			
			--sync pulse in sync phase
			if (HCnt >= H_800_480p_AV_FP-1) and (HCnt < H_800_480p_AV_FP_S-1) then -- one cycle earlier (registered)
				hs <= '0';
				if (VCnt >= V_800_480p_AV_FP) and (VCnt < V_800_480p_AV_FP_S) then
					vs <= '0';
				else
					vs <= '1';
				end if;				
			else
				hs <= '1';
			end if;
			
			--video data enable
			if ((HCnt = H_800_480p_AV_FP_S_BP - 1 and (VCnt = V_800_480p_AV_FP_S_BP - 1 or VCnt < V_800_480p_AV - 1)) or -- first pixel in frame
				 (HCnt < H_800_480p_AV - 1 and VCnt < V_800_480p_AV)) then
				vde <= '1';
			else
				vde <= '0';
			end if;
			
		end if;

end process;

--process (PCLK_I)
--begin
--	if Rising_Edge(PCLK_I) then
        HS_O <= 	not hs	when H_800_480p_POL else
                hs;
        VS_O <= 	not vs	when V_800_480p_POL else
                vs;
        VDE_O <= vde;
    
    
    
    
    
        -- generate colors that divides the display into three vertical stripes
        --BLUE					YELLOW				RED
        --(0, 0, x"FF")		(x"FF", x"FF", 0)	(x"FF", 0, 0)
    
--        red_reg		<= x"FF" when HCnt > H_800_480p_AV/3 else x"00";
--        green_reg	<= x"FF" when HCnt > H_800_480p_AV/3 and HCnt < 2*H_800_480p_AV/3 else x"00";
--        blue_reg 	<= x"FF" when HCnt < H_800_480p_AV/3 else x"00";


        red_reg		<= x"FF" when HCnt > H_800_480p_AV/3 and HCnt < H_800_480p_AV else x"00";
        green_reg	<= x"FF" when HCnt > H_800_480p_AV/3 and HCnt < 2*H_800_480p_AV/3 else x"00";
        blue_reg 	<= x"FF" when HCnt < H_800_480p_AV/3 and HCnt > 0 else x"00";
--    red_reg <= x"FF" when HCnt < H_800_480p_AV/2 and HCnt > 0 else x"00";
--    green_reg <= x"FF" when HCnt < H_800_480p_AV/2 and HCnt > 0 else x"00";
--    blue_reg <= x"FF" when HCnt < H_800_480p_AV/2 and HCnt > 0 else x"00";
--	end if;
        
--end process;

	R_O <=  red_reg;
	G_O <=  green_reg;
	B_O <=  blue_reg;	
	
end Behavioral;

